/*
 * him-arabic.cc
 * This file is part of him-arabic
 *
 * Copyright (C) 2007 Mohammed Sameer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "him-arabic.hh"
#include "plugin.hh"
#include "utils.hh"

void him_arabic_language_settings_changed(HildonIMPlugin *pl, gint index) {
  // Empty
}

void him_arabic_input_mode_changed(HildonIMPlugin *pl) {
  // Empty
}

void him_arabic_clear(HildonIMPlugin *pl) {
  // Empty
}

void him_arabic_enable(HildonIMPlugin *pl, gboolean init) {
  HimArabic *har = HIM_ARABIC(pl);
  HimArabicPriv *priv = HIM_ARABIC_GET_PRIVATE(har);
  if (init == TRUE) {
    hildon_im_ui_button_set_sensitive(priv->ui, HILDON_IM_BUTTON_MODE_A, TRUE);
    hildon_im_ui_button_set_sensitive(priv->ui, HILDON_IM_BUTTON_MODE_B, TRUE);
    hildon_im_ui_button_set_sensitive(priv->ui, HILDON_IM_BUTTON_SPECIAL_CHAR, FALSE);
    hildon_im_ui_button_set_toggle(priv->ui,
				   HILDON_IM_BUTTON_MODE_A, TRUE);
    hildon_im_ui_button_set_toggle(priv->ui,
    				   HILDON_IM_BUTTON_MODE_B, TRUE);
    hildon_im_ui_button_set_label(priv->ui,
				  HILDON_IM_BUTTON_MODE_A, "EN");
    hildon_im_ui_button_set_label(priv->ui,
				  HILDON_IM_BUTTON_MODE_B, "Shift");
  }

  hildon_im_ui_send_communication_message(priv->ui,
					  HILDON_IM_CONTEXT_DIRECT_MODE);
}

void him_arabic_disable(HildonIMPlugin *pl) {
  // Empty
}

void him_arabic_enter(HildonIMPlugin *pl) {
  hildon_im_ui_send_communication_message(HIM_ARABIC_GET_PRIVATE(HIM_ARABIC(pl))->ui,
					  HILDON_IM_CONTEXT_HANDLE_ENTER);
}

void him_arabic_tab(HildonIMPlugin *pl) {
  hildon_im_ui_send_communication_message(HIM_ARABIC_GET_PRIVATE(HIM_ARABIC(pl))->ui,
					  HILDON_IM_CONTEXT_HANDLE_TAB);
}

void him_arabic_backspace(HildonIMPlugin *pl) {
  hildon_im_ui_send_communication_message(HIM_ARABIC_GET_PRIVATE(HIM_ARABIC(pl))->ui,
					  HILDON_IM_CONTEXT_HANDLE_BACKSPACE);
}

void him_arabic_save_data(HildonIMPlugin *pl) {
  // Empty
}

void him_arabic_language(HildonIMPlugin *pl) {
  // Empty
}

void him_arabic_mode_a(HildonIMPlugin *pl) {
  HIM_ARABIC(pl)->mode_a = hildon_im_ui_button_get_active(HIM_ARABIC_GET_PRIVATE(HIM_ARABIC(pl))->ui, HILDON_IM_BUTTON_MODE_A);
  // mode_a is always for English.
  if (HIM_ARABIC(pl)->mode_a) {
    him_arabic_reset_layouts(HIM_ARABIC(pl), "en");
  }
  else {
    him_arabic_reset_layouts(HIM_ARABIC(pl));
  }
}

void him_arabic_mode_b(HildonIMPlugin *pl) {
  HIM_ARABIC(pl)->mode_b = hildon_im_ui_button_get_active(HIM_ARABIC_GET_PRIVATE(HIM_ARABIC(pl))->ui, HILDON_IM_BUTTON_MODE_B);
  if (HIM_ARABIC(pl)->mode_a) {
    him_arabic_reset_layouts(HIM_ARABIC(pl), "en");
  }
  else {
    him_arabic_reset_layouts(HIM_ARABIC(pl));
  }
}

void him_arabic_settings_changed(HildonIMPlugin *pl, const gchar *key, const GConfValue *val) {
  if ((key) && (!strcmp(key, "/apps/osso/inputmethod/hildon-im-languages/current"))) {
    him_arabic_reset_layouts(HIM_ARABIC(pl));
    hildon_im_ui_button_set_toggle(HIM_ARABIC_GET_PRIVATE(HIM_ARABIC(pl))->ui,
				   HILDON_IM_BUTTON_MODE_A, TRUE);
  }
}

void him_arabic_client_widget_changed(HildonIMPlugin *pl) {
  // Todo: Check the Auto capitalization.
  // Empty
}

void him_arabic_reset_layouts(HimArabic *har, gchar *lang) {
  HimArabicPriv *priv = HIM_ARABIC_GET_PRIVATE(har);
  // Do we have a loaded layout for it ?
  const gchar *active = lang ? lang : hildon_im_ui_get_active_language(priv->ui);
  std::map<std::string, std::map<LayoutType, GtkWidget *> >::iterator iter = har->layouts->find(active);
  if (iter == har->layouts->end()) {
    gchar** parts = g_strsplit(active, "_", 0);
    iter = har->layouts->find(parts[0]);
    if (iter == har->layouts->end()) {
      // Let's try to load it ?
      std::string which;
      std::string xml = get_data(active, parts[0], which);
      if (xml.size() == 0) {
	// failed.
	him_arabic_signal_error(har, "Failed to load the data file.");
      }
      else {
	Data data(xml);
	if (!data) {
	  // Failed. inform the user.
	  him_arabic_signal_error(har, "Failed to parse the data file.");
	}
	else {
	  // Get the keys, Construct the layout and activate it.
	  try {
	    him_arabic_draw_layout(har, LAYOUT_DEFAULT, data.get_layout(LAYOUT_DEFAULT), which);
	    him_arabic_draw_layout(har, LAYOUT_B, data.get_layout(LAYOUT_B), which);
	  }
	  catch (std::string& error) {
	    // Do nothing.
	  }
	  // activate it.
	  iter = har->layouts->find(which);
	  him_arabic_activate_layout(har, iter->second);
	}
      }
    }
    else {
      // activate layout
      him_arabic_activate_layout(har, iter->second);
    }
    g_strfreev(parts);
  }
  else {
    // activate layout
    him_arabic_activate_layout(har, iter->second);
  }
}

void him_arabic_signal_error(HimArabic *har, const std::string& err) {
  osso_system_note_infoprint(HIM_ARABIC_GET_PRIVATE(har)->ui->osso, err.c_str(), NULL);
}
